#!/usr/bin/perl

# ./convertRRD.php -or mrtg5443.rrd -nr mrtg5443new.rdd -ds ds0:traffic_in -ds ds1:traffic_out -od /opt/home/ssorli -nd /opt/home/ssorli -b

use strict;
use Getopt::Long;
use File::Copy;

use vars qw( $oldrrd $newrrd @ds %ds $backup $force @xml $help );
use vars qw( $newdir $olddir $cactiuser );
use vars qw( $rrdump $rrrestore );


if ($#ARGV < 3) {
 &usage();
 exit;
}

GetOptions( "or=s"		=>	\$oldrrd,
            "nr=s"		=>	\$newrrd,
			"ds=s"		=>	\@ds,
			"od=s"		=>  \$olddir,
			"nd=s"		=>	\$newdir,
			"b"			=>	\$backup,
			"f"			=>  \$force,
			"h"			=>	\$help,
		  );

&usage() if( $help );

%ds = ();

$cactiuser = "cactiuser:cactiuser";
$rrdump = "/opt/rrdtool/bin/rrdtool dump";
$rrrestore = "/opt/rrdtool/bin/rrdtool restore";

die ("No such rrd: $olddir/$oldrrd!\n" ) unless ( -e "$olddir/$oldrrd" and -w $newdir );

foreach ( @ds ) {
	/(\w+):(\w+)/;
	$ds{$1} = $2;
}

if( $backup ) {
	# Backup the existing rrd
	if( -e "$newdir/$newrrd.bak" ) 	{
		if ( $force ) {
			unlink( "$newdir/$newrrd.bak" );
		} else {
			my $answer;
			while( !$answer ) {
				print( "Remove existing backup? [y] " );
				$answer = <STDIN>;
				chop( $answer );
				$answer="y" if( !$answer );
				if( $answer =~ /^y$/i ) {
					# Whack the existing backup
					die( "Can't remove $newdir/$newrrd.bak: $!\n" ) unless( -w "$newdir/$newrrd.bak" );
					unlink( "$newdir/$newrrd.bak" );
				} elsif( $answer =~ /^n$/i ) {
					# Don't whack it
					last;
				} else {
					print( "Please enter y or n.\n" );
				}
			}
		}
	}

	# Actually do the backup
	move( "$newdir/$newrrd", "$newdir/$newrrd.bak" ) || die( "Can't make backup: $!\n" );
} else {
	# No backup - just kill it
	unlink( "$newdir/$newrrd" );
}

# With a backup made (or ignored), we can do the dump and such

# Dump the rrd
print( "Dumping $oldrrd.\n" );
@xml = `$rrdump $olddir/$oldrrd`;

# Swap our ds names
foreach ( @xml ) {
	if( /<name>\s+(\w+)\s+<\/name>/ ) {
		my $d = $1;
		print( "Found $d -> " );
		if( $ds{$d} ) {
			s/$1/$ds{$d}/;
			print( "Replaced with " . $ds{$d} . "\n" );
		} else {
			print( "Ignoring.\n" );
		}
	}
}

# Write it out
print( "Writing XML..." );
open( OUT, ">$newdir/$newrrd.xml" );
foreach( @xml ) {
	print( OUT $_ );
}
print( "done.\n" );

# Import it
print( "Importing..." );
`$rrrestore $newdir/$newrrd.xml $newdir/$newrrd`;
print( "done.\n" );
print( "Deletting XML...\n" );
unlink( "$newdir/$newrrd.xml" );

# Change perms
`chown $cactiuser $newdir/$newrrd`;
print( "Exiting.\n" );

sub usage {
	print( "Usage: convertRRD.pl -or <oldrrd> -nr <newrrd> [ -ds <old1:new1> -ds <old2:new2> ... -ds <oldn:newn> ] -od <olddir> -b\n" );
	print( "  -or : old rrd file\n" );
	print( "  -nr : new rrd file\n" );
	print( "  -ds : ds conversion pair\n" );
	print( "  -od : where to find -or\n" );
	print( "  -nd : where to put -nr\n" );
	print( "  -f  : force to put -f\n" );
	print( "  -b  : backup the -nr before importing\n" );
	exit 1;
}
