<?php
include_once("device_db.php");
include_once("././lib/utility.php");
include_once("././lib/api_data_source.php");
include_once("././lib/api_graph.php");
include_once("././lib/snmp.php");
include_once("././lib/data_query.php");
include_once("././lib/api_device.php");

/*
 * Function update data device
 *
 * @arg $hostId - (num)
 * @arg $template_id - (num)
 * @arg $description - (str)
 * @arg $fqdn - (str)
 * @arg $community - (str)
 * @arg $snmp_ver [1,2,3] - (num)
 * @arg $disable [1]disabled, [0]enabled  - (num)
 * @arg $snmp_username - (str)
 * @arg $snmp_password - (str)
 * @arg $snmp_port [161] - (num)
 * @arg $snmp_timeout [500] - (num)
 * @arg $disable
 * @arg $avail
 * @arg $ping_method
 * @arg $ping_port
 * @arg $ping_timeout
 * @arg $ping_retries
 * @arg $notes
 * @arg $snmp_auth_protocol
 * @arg $snmp_priv_passphrase
 * @arg $snmp_priv_protocol
 * @arg $snmp_context
 * @arg &$errors
 * @returns - $host_id - (num)
 * @Exception - Input validation, DB validation
 * @Waring - Record exist
 */
function api_device_write_Device($hostId, $template_id, $description, $fqdn, $community, $snmp_ver,
$snmp_username, $snmp_password, $snmp_port, $snmp_timeout, $disable = 0, $avail = 1, $ping_method = 3,
$ping_port= 23, $ping_timeout = 500, $ping_retries = 2, $notes = "", $snmp_auth_protocol = "", $snmp_priv_passphrase = "",
$snmp_priv_protocol = "", $snmp_context = "", &$errors)
{
	if (!is_numeric($ping_retries) && ($ping_retries <= 0)) {
		catchException($ping_retries, "ping_retries", "api_device_write_Device", "Invalid Ping Retries: ($ping_retries)", $errors);
	}
	if ($ping_port <= 1 || $ping_port > 65534) {
		catchException($ping_port, "ping_port", "api_device_write_Device", "Invalid port. Valid values are from 1-65534", $errors);
	}
	if ($snmp_ver != "1" && $snmp_ver != "2" && $snmp_ver != "3") {
		catchException($snmp_ver, "snmp_ver", "api_device_write_Device", "Invalid snmp version", $errors);
	}
	if ($snmp_port <= 1 || $snmp_port > 65534) {
		catchException($snmp_port, "snmp_port", "api_device_write_Device", "Invalid port. Valid values are from 1-65534", $errors);
	}
	if ($snmp_timeout <= 0 || $snmp_timeout > 20000) {
		catchException($snmp_timeout, "snmp_timeout", "api_device_write_Device", "Invalid timeout.  Valid values are from 1 to 20000", $errors);
	}
	if ($disable != "1" && $disable != "0") {
		catchException($disable, "disable", "api_device_write_Device", "Value not valid", $errors);
	}
	if ($disable == "0") $disable = ""; else $disable = "on";
	if(!empty($errors)){
		return 1;
	}
	
	if ($hostId > 0){  //update host data
		$host = getHostByID($hostId, $errors);
		if (!empty($errors)){
			catchException("", "", "api_device_write_Device", "Error in main function", $errors);
			return 1;
		}
	}

	getHostTemplateByID($template_id, $errors);
	if(!empty($errors)){
		catchException("", "", "api_device_write_Device", "Error in main function", $errors);
		return 1;
	}

	$host = getHostByDescription($description, false, $errors);
	if (!empty($errors)) { //this is only waring
		catchException("", "", "api_device_write_Device", "Error in main function", $errors);
		return 1;
	}
	if (isset($host) && !empty($host)) {
		return $host;
	}

	$host = getHostByfqdn($fqdn, false, $errors); //id, hostname
	if (!empty($errors)) { //this is only waring
		catchException("", "", "api_device_write_Device", "Error in main function", $errors);
		return 1;
	}
	if (isset($host) && !empty($host)) {
		return $host;
	}

	/*	$avail        = 1;
	 $ping_method  = 3;
	 $ping_port    = 23;
	 $ping_timeout = 500;
	 $ping_retries = 2;
	 $notes         = "";
	 $snmp_auth_protocol   = "";
	 $snmp_priv_passphrase = "";
	 $snmp_priv_protocol   = "";
	 $snmp_context         = "";*/

	$host_id = api_device_save($hostId, $template_id, $description, $fqdn,
	$community, $snmp_ver, $snmp_username, $snmp_password,
	$snmp_port, $snmp_timeout, $disable, $avail, $ping_method,
	$ping_port, $ping_timeout, $ping_retries, $notes,
	$snmp_auth_protocol, $snmp_priv_passphrase,
	$snmp_priv_protocol, $snmp_context);

	if (!isset($host_id) || empty($host_id) || is_error_message()) {
		catchException("", "", "api_device_write_Device", "Failed to add this device", $errors);
		return 1;
	} else {
		$host["id"] = $host_id;
		$host["hostname"] = $fqdn;
		$host["description"] = $description;
	}
	return $host;
}

/* Function update FQDN - ip or full domain name of device
 *
 * @arg $fqdn - (str)
 * @arg $description - (str)
 * @returns - $hostId - (num)
 * @Exception - Input validation, DB validation, record must exist
 * @Waring - Non
 */
function api_device_update_FQDN($fqdn, $description, &$errors) {
	$host = getHostByDescription($description, true, $errors); //id, description
	if (!empty($errors)) {
		catchException("", "", "api_device_update_FQDN", "Error in main function", $errors);
		return 1;
	}
	updateHostFQDN($fqdn, $host["id"], $errors);
	if (!empty($errors)) {
		catchException("", "", "api_device_update_FQDN", "Error in main function", $errors);
		return 1;
	}
	return $host; //id, hostname
}

/* Function update description of device
 *
 * @arg $description - (str)
 * @arg $fqdn - (str)
 * @returns - $hostId - (num)
 * @Exception - Input validation, DB validation, record must exist
 * @Waring - Non
 */
function api_device_update_Description($description, $fqdn, &$errors) {
	$errors = array();

	$host = getHostByfqdn($fqdn, true, $errors);
	if (!empty($errors)) {
		catchException("", "", "api_device_update_Description", "Error in main function", $errors);
		return 1;
	}
	updateHostDescription($description, $host["id"], $errors);
	if (!empty($errors)) {
		catchException("", "", "api_device_update_Description", "Error in main function", $errors);
		return 1;
	}
	return $host; //id, hostname
}

/*
 * Function get Host Template id By Name
 *
 * @arg $hostTemplateName - (str)
 * @returns - $hostTemplateId - (num)
 * @Exception - Input validation, DB validation, record must exist
 * @Waring - Non
 */
function api_device_get_Host_Template_By_Name($hostTemplateName, &$errors)
{
	$hostTemplate = getHostTemplateByName($hostTemplateName, $errors); //id, name
	if(!empty($errors)){
		catchException("", "", "api_device_get_Host_Template_By_Name", "Error in main function", $errors);
		return 1;
	}
	return $hostTemplate;//id, name
}

/* Function get list records Host Template
 * @arg - Non
 * @returns - $hostTemplate[id, name] - (Array(Array))
 * @Exception - records must exist
 * @Waring - Non
 */
function api_device_get_Host_Templates(&$errors)
{
	$hostTemplatesReturn = getHostTemplates($errors); //array id, name
	if(!empty($errors)){
		catchException("", "", "api_device_get_Host_Templates", "Error in main function", $errors);
		return 1;
	}
	return $hostTemplatesReturn; //array id, name
}

/* Function get all Communities
 * @arg - Non
 * @returns - $communities[snmp_community] - (Array(Array))
 * @Exception - records must exist
 * @Waring - Non
 */
function api_device_get_Communities(&$errors)
{
	$communities = getHostCommunities($errors);
	if(!empty($errors)){
		catchException("", "", "api_device_get_Communities", "Error in main function", $errors);
		return 1;
	}
	return $communities;
}

/* Function get list of all Hosts
 *
 * @arg - Non
 * @returns - $hosts["id"], $hosts["hostname"] - (long)
 * @Exception - No DB data
 * @Waring - Non
 */
function api_device_get_Hosts(&$errors)
{
	$hosts = getHosts($errors);
	if(!empty($errors)){
		catchException("", "", "api_device_get_Hosts", "Error in main function", $errors);
		return 1;
	}
	return $hosts;
}

/* Function get host id selected by host name
 *
 * @arg - $hostName - host name (str)
 * @returns - ["id", "hostname"]
 * @Exception - Input validation, No DB data
 * @Waring - Non
 */
function api_device_get_By_FQDN($hostName, &$errors)
{
	$host = getHostByfqdn($hostName, true, $errors); //id, hostname, description
	if(!empty($errors)){
		catchException("", "", "api_device_get_By_FQDN", "Error in main function", $errors);
		return 1;
	}
	return $host;
}

?>
