<?php
include_once("tree_db.php");
include_once("././lib/tree.php");
include_once("././lib/api_tree.php");
//include_once("././lib/api_automation_tools.php");

/* Function add tree to the cacti menu
 *
 * @arg $name - (str)
 * @arg $sortMethod - (str)
 * @returns - $treeId - (num)
 * @Exception - Invalid inputs, DB save
 * @Waring - Record exist
 */
function api_tree_add_tree($name, $sortMethod, &$errors){

	$tree = getTreeByName($name, false, $errors); //id, name
	if(!empty($errors)){ //this is only waring
		catchException($tree, "tree", "api_tree_add_tree", "Error in main function", $errors);
		return 1;
	}
	if (isset($tree) && !empty($tree)) { //warnig
		return $tree; //id, name
	}

	$tree = addTree($name, $sortMethod, $errors);
	if(!empty($errors)){
		catchException("", "tree", "api_tree_add_tree", "Error in main function", $errors);
		return 1;
	}
	return $tree; //id, name
}

/* Function add node header to cacti menu
 *
 * @arg $name - (str)
 * @arg $parentNode - (num)
 * @arg $treeId - (num)
 * @returns - $nodeId - (long)
 * @Exception - Invalid inputs, DB save
 * @Waring - Record exist
 */
function api_tree_add_node_header($name, $parentNode, $treeId, &$errors){

	$treeNode = getTreeNodeByName($treeId, $name, false, $errors); //record exist
	if(!empty($errors)){ //this is only waring
		catchException($treeNodes, "treeNodes", "api_tree_add_node_header", "Error in main function", $errors);
		return 1;
	}
	if (isset($treeNode) && !empty($treeNode)) {
		return $treeNode;
	}

	global $config;
	$parent_order_key = db_fetch_cell("select order_key from graph_tree_items where id = $parentNode");
	$search_key = substr($parent_order_key, 0, (tree_tier($parent_order_key) * CHARS_PER_TIER));
	/* Duplicate header check */
	if ((sizeof(db_fetch_assoc("select id from graph_tree_items where title='$name' and graph_tree_id=" . $treeId . " and order_key like '$search_key" . str_repeat('_', CHARS_PER_TIER) . str_repeat('0', (MAX_TREE_DEPTH * CHARS_PER_TIER) - (strlen($search_key) + CHARS_PER_TIER)) . "'")) > 0)) {
		return db_fetch_cell("select id, title from graph_tree_items where title='$name' and graph_tree_id= " .$treeId . " and order_key like '$search_key" . str_repeat('_', CHARS_PER_TIER) . str_repeat('0', (MAX_TREE_DEPTH * CHARS_PER_TIER) - (strlen($search_key) + CHARS_PER_TIER)) . "'");
	}
	
	# $nodeId could be a Header Node, a Graph Node, or a Host node.
	$treeNodeId = api_tree_item_save(0, $treeId, 1, $parentNode, $name, 0, 0, 0, 1, 1, false); 
	if(!isset($treeNodeId) || empty($treeNodeId)){
		catchException("", "treeNodeId", "api_tree_add_node_header", "Error in main function", $errors);
		return 1;
	}
	$treeNode["id"] = $treeNodeId;
	$treeNode["title"] = $name;
	return $treeNode;//id, title
}

/* Function add node host to cacti menu
 *
 * @arg $name - (str)
 * @arg $parentNode - (num)
 * @arg $treeId - (num)
 * @arg $hostId - (num)
 * @arg $hostGroupStyle (Host group styles: 1 = Graph template, 2 = Data Query Index) - (num)
 * @returns - $nodeId - (long)
 * @Exception - Invalid inputs, DB save
 * @Waring - Record exist
 */
function api_tree_add_node_host($name,
$parentNode,
$treeId,
$hostId,
$hostGroupStyle, &$errors){

	$treeNode = getTreeNodeByName($treeId, $name, false, $errors); //record exist
	if(!empty($errors)){ //this is only waring
		catchException($treeNodes, "treeNodes", "api_tree_add_node_host", "Error in main function", $errors);
		return 1;
	}
	if (isset($treeNode) && !empty($treeNode)) {
		return $treeNode;
	}

	if ($hostGroupStyle != 1 && $hostGroupStyle != 2)
	{
		catchException($hostGroupStyle, "hostGroupStyle", "api_tree_add_node_host", "Host Group Style must be 1 or 2 (Graph Template or Data Query Index)", $errors);
	}

	getParentNode($treeId, $parentNode, &$errors); //host must exist to add them to tree
	getHostByID($hostId, $errors);
	$parentNodeExist = getParentNodeByHostId($treeId, $hostId, false, $errors); //record must not exist
	if (!empty($errors)) //warning
	{
		catchException("", "", "api_tree_add_node_host", "Error in main function", $errors);
		return 1;
	}
	if (isset($parentNodeExist) && !empty($parentNodeExist)) {
		return $parentNodeExist;//id, title
	}

	# $nodeId could be a Header Node, a Graph Node, or a Host node.
	$nodeId = api_tree_item_save(0, $treeId, 3, $parentNode, '', 0, 0, $hostId, $hostGroupStyle, 1, false);
	if(!isset($nodeId) || empty($nodeId)){
		catchException("", "nodeId", "api_tree_add_node_host", "Error in main function", $errors);
		return 1;
	}
	$node["id"] = $nodeId;
	$node["title"] = $name;
	return $node;//id, title
}

/* Function add node graph to cacti menu
 *
 * @arg $parentNode - (num)
 * @arg $treeId - (num)
 * @arg $graphId - (num)
 * @arg $rra_id - (num)
 * @returns - $nodeId - (num)
 * @Exception - Invalid inputs, DB save
 * @Waring - Non
 */
function api_tree_add_node_graph($parentNode, $treeId, $graphId, $rra_id, &$errors){
	
	global $config;
	$parent_order_key = db_fetch_cell("select order_key from graph_tree_items where id=$parentNode");
	/* duplicate graph check */
	$search_key = substr($parent_order_key, 0, (tree_tier($parent_order_key) * CHARS_PER_TIER));
	if (sizeof(db_fetch_assoc("select id from graph_tree_items where local_graph_id= " . $graphId . " and graph_tree_id = " . $treeId . " and order_key like '$search_key" . str_repeat('_', CHARS_PER_TIER) . str_repeat('0', (MAX_TREE_DEPTH * CHARS_PER_TIER) - (strlen($search_key) + CHARS_PER_TIER)) . "'")) > 0) {
		return db_fetch_cell("select id from graph_tree_items where local_graph_id= " . $graphId . " and graph_tree_id= " . $treeId . " and order_key like '$search_key" . str_repeat('_', CHARS_PER_TIER) . str_repeat('0', (MAX_TREE_DEPTH * CHARS_PER_TIER) - (strlen($search_key) + CHARS_PER_TIER)) . "'");
	}

	$nodeId = api_tree_item_save(0, $treeId, 2, $parentNode, '', $graphId, $rra_id, 0, 1, 1, false, $errors);

	if(!isset($nodeId) || empty($nodeId)){
		catchException("", "nodeId", "api_tree_add_node_graph", "Error in main function", $errors);
		return 1;
	}
	$node["id"] = $nodeId;
	//$node["title"] = $name;
	return $node;
}

/* Function get Tree Nodes to cacti menu
 *
 * @arg $treeId - (num)
 * @returns - $treeNodes["id"], $treeNodes["title"] - (Array(Array))
 * @Exception - Input validation, No DB data
 * @Waring - Non
 */
function api_tree_get_TreeNodes($treeId, &$errors)
{
	$treeNodes = getTreeNodes($treeId, $errors);
	if(!empty($errors)){
		catchException("", "treeNodes", "api_tree_get_TreeNodes", "Error in main function", $errors);
		return 1;
	}
	return $treeNodes;
}

/* Function get list of all Trees
 *
 * @arg $name - (str)
 * @returns - $tree["id"], $tree["name"] - (Array(Array))
 * @Exception - Input validation, No DB data
 * @Waring - Non
 */
function api_tree_get_Trees(&$errors)
{
	$trees = getTrees($errors);
	if(!empty($errors)){
		catchException("", "trees", "api_tree_get_Trees", "Error in main function", $errors);
		return 1;
	}
	return $trees;
}

/* Function get Tree id by Name
 *
 * @arg $name - (str)
 * @returns - $treeId - (num)
 * @Exception - Input validation, No DB data
 * @Waring - Non
 */
function api_tree_get_Tree_By_Name($name, &$errors)
{
	$tree = getTreeByName($name, true, $errors);
	if(!empty($errors)){
		catchException("", "tree", "api_tree_get_Tree_By_Name", "Error in main function", $errors);
		return 1;
	}
	return $tree;
}

/* Function get Tree Node id by Name
 *
 * @arg $treeId - (num)
 * @arg $title - (str)
 * @returns - $treeNodeId - (num)
 * @Exception - Input validation, No DB data
 * @Waring - Non
 */
function api_tree_get_Tree_Node_By_Name($treeId, $title, &$errors)
{
	$treeNode = getTreeNodeByName($treeId, $title, true, $errors);//id, title 
	if(!empty($errors)){
		catchException("", "treeNode", "api_tree_get_Tree_Node_By_Name", "Error in main function", $errors);
		return 1;
	}
	return $treeNode;
}

?>