<?php
/*
  Written By Sergej Sorli <sergej.sorli[-a-t-]arnes.si>
 +-------------------------------------------------------------------------+
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
 | cacti: a php-based graphing solution                                    |
 +-------------------------------------------------------------------------+
*/


include("./include/global.php");
include_once("./lib/functions.php");
include_once("./soap/auth_login_db.php");
include_once("./soap/device_bl.php");
include_once("./soap/graph_bl.php");
include_once("./soap/tree_bl.php");
include_once("./soap/data_bl.php");
include_once("./soap/error_data.php");

function pc_authenticate_user($username, $password){
	$is_valid = auth_login_db($username, $password, $_SERVER["REMOTE_ADDR"]);

	if($is_valid){
		return true;
	}else{
		return false;
	}
}

class pc_SOAP_return_soap {
	private $authenticated;

	public function __construct(){
		$this->authhenticated = false;
	}

	/*
	 * Autenticate user
	 * 
	 * if autentication is ok then variable authenticated is true
	 */
	public function authenticate_user($args){
		if(! pc_authenticate_user($args->username, $args->password)){
			return;
		}
		$this->authenticated = true;
	}

	/*
	 * Function add device to record if not exist
	 *
	 * @arg $template_id - (num)
	 * @arg $description - (str)
	 * @arg $fqdn - (str)
	 * @arg $community - (str)
	 * @arg $snmp_ver [1,2,3] - (num)
	 * @arg $disable [1]disabled, [0]enabled  - (num)
	 * @arg $snmp_username - (str)
	 * @arg $snmp_password - (str)
	 * @arg $snmp_port [161] - (num)
	 * @arg $snmp_timeout [500] - (num)
	 * @arg $disable
	 * @arg $avail
	 * @arg $ping_method
	 * @arg $ping_port
	 * @arg $ping_timeout
	 * @arg $ping_retries
	 * @arg $notes
	 * @arg $snmp_auth_protocol
	 * @arg $snmp_priv_passphrase
	 * @arg $snmp_priv_protocol
	 * @arg $snmp_context
	 * @returns - $host_id - (num)
	 * @Exception - Input validation, DB validation
	 * @Waring - Record exist
	 */
	public function return_ID_add_Device($template_id, $description, $fqdn, $community, 
		$snmp_ver, $snmp_username, $snmp_password, $snmp_port, $snmp_timeout, $disable, $avail, 
		$ping_method, $ping_port, $ping_timeout, $ping_retries, $notes, $snmp_auth_protocol, 
		$snmp_priv_passphrase, $snmp_priv_protocol, $snmp_context)
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_add_Device","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_device_write_Device(0, $template_id, $description, $fqdn, $community, 
		$snmp_ver, $snmp_username, $snmp_password, $snmp_port, $snmp_timeout, $disable, $avail, 
		$ping_method, $ping_port, $ping_timeout, $ping_retries, $notes, $snmp_auth_protocol, 
		$snmp_priv_passphrase, $snmp_priv_protocol, $snmp_context, $errors); //value
		
		return soapResponse($returnResult["id"], $errors);
	}

	/*
	 * Function update data device
	 * 
	 * @arg $hostId - (num)
	 * @arg $template_id - (num)
	 * @arg $description - (str)
	 * @arg $fqdn - (str)
	 * @arg $community - (str)
	 * @arg $snmp_ver [1,2,3] - (num)
	 * @arg $disable [1]disabled, [0]enabled  - (num)
	 * @arg $snmp_username - (str)
	 * @arg $snmp_password - (str)
	 * @arg $snmp_port [161] - (num)
	 * @arg $snmp_timeout [500] - (num)
	 * @arg $disable
	 * @arg $avail
	 * @arg $ping_method
	 * @arg $ping_port
	 * @arg $ping_timeout
	 * @arg $ping_retries
	 * @arg $notes
	 * @arg $snmp_auth_protocol
	 * @arg $snmp_priv_passphrase
	 * @arg $snmp_priv_protocol
	 * @arg $snmp_context
	 * @returns - $host_id - (num)
	 * @Exception - Input validation, DB validation
	 * @Waring - Record exist
	 */
	public function return_ID_update_Device($hostId, $template_id, $description, $fqdn, $community, 
		$snmp_ver, $snmp_username, $snmp_password, $snmp_port, $snmp_timeout, $disable, $avail, 
		$ping_method, $ping_port, $ping_timeout, $ping_retries, $notes, $snmp_auth_protocol, 
		$snmp_priv_passphrase, $snmp_priv_protocol, $snmp_context)
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_update_Device","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_device_write_Device($hostId, $template_id, $description, $fqdn, $community, 
		$snmp_ver, $snmp_username, $snmp_password, $snmp_port, $snmp_timeout, $disable, $avail, 
		$ping_method, $ping_port, $ping_timeout, $ping_retries, $notes, $snmp_auth_protocol, 
		$snmp_priv_passphrase, $snmp_priv_protocol, $snmp_context, $errors); //value
		
		return soapResponse($returnResult["id"], $errors);
	}
	
	/*
	 * Function update FQDN - ip or full domain name of device
	 *
	 * @arg $fqdn - (str)
	 * @arg $description - (str)
	 * @returns - $hostId - (num)
	 * @Exception - Input validation, DB validation, record must exist
	 * @Waring - Non
	 */
	public function return_ID_update_FQDN($fqdn, $description) {
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_update_FQDN","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_device_update_FQDN($fqdn, $description, $errors); //id, hostname
		return soapResponse($returnResult["id"], $errors);
	}

	/*
	 * Function update description of device
	 *
	 * @arg $description - (str)
	 * @arg $fqdn - (str)
	 * @returns - $hostAddresseId - (num)
	 * @Exception - Input validation, DB validation, record must exist
	 * @Waring - Non
	 */
	public function return_ID_update_Description($description, $fqdn) {
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_update_Description","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_device_update_Description($description, $fqdn, $errors); //id, hostname
		return soapResponse($returnResult["id"], $errors);
	}

	/*
	 * Function get Host Template id By Name
	 *
	 * @arg $hostTemplateName - (str)
	 * @returns - $hostTemplateId - (num)
	 * @Exception - Input validation, DB validation, record must exist
	 * @Waring - Non
	 */
	public function return_ID_get_Host_Template_By_Name($hostTemplateName)
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_get_Host_Template_By_Name","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_device_get_Host_Template_By_Name($hostTemplateName, $errors);//id, name
		return soapResponse($returnResult["id"], $errors);
	}

	/* Function get list records Host Template
	 * @arg - Non
	 * @returns - $hostTemplate[id, name] - (Array(Array))
	 * @Exception - records must exist
	 * @Waring - Non
	 */
	public function return_list_IDName_get_Host_Templates()
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_list_IDName_get_Host_Templates","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_device_get_Host_Templates($errors);//array id, name 

		return soapResponse($returnResult, $errors);
	}

	/* Function get all Communities
	 * @arg - Non
	 * @returns - $communities[name] - (Array(Array))
	 * @Exception - records must exist
	 * @Waring - Non
	 */
	public function return_list_snmp_community_get_Communities()
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_list_snmp_community_get_Communities","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$i = 1;	
		$returnResult = api_device_get_Communities($errors);

		return soapResponse($returnResult, $errors);
	}

	/* Function get list of all Hosts
	 *
	 * @arg - Non
	 * @returns - $hosts["id"], $hosts["hostname"] - (long)
	 * @Exception - No DB data
	 * @Waring - Non
	 */
	public function return_list_IDHostname_get_Hosts()
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_list_IDHostname_get_Hosts","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_device_get_Hosts($errors); //Array id, hostname

		return soapResponse($returnResult, $errors);
	}

	//#####################################


	/* Function add tree to the cacti menu
	 *
	 * @arg $name - (str)
	 * @arg $sortMethod - (str)
	 * @returns - $treeId - (num)
	 * @Exception - Invalid inputs, DB save
	 * @Waring - Record exist
	 */
	public function return_ID_add_tree($name, $sortMethod){

		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_add_tree","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_tree_add_tree($name, $sortMethod, $errors); //id, name
		return soapResponse($returnResult["id"], $errors);
	}

	/* Function add node header to cacti menu
	 *
	 * @arg $name - (str)
	 * @arg $parentNode - (num)
	 * @arg $treeId - (num)
	 * @returns - $nodeId - (long)
	 * @Exception - Invalid inputs, DB save
	 * @Waring - Record exist
	 */
	public function return_ID_add_node_header($name,
	$parentNode,
	$treeId){

		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_add_node_header","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_tree_add_node_header($name, $parentNode, $treeId, $errors);
		return soapResponse($returnResult["id"], $errors);
	}

	/* Function add node host to cacti menu
	 *
	 * @arg $name - (str)
	 * @arg $parentNode - (num)
	 * @arg $treeId - (num)
	 * @arg $hostId - (num)
	 * @arg $hostGroupStyle (Host group styles: 1 = Graph template, 2 = Data Query Index) - (num)
	 * @returns - $nodeId - (long)
	 * @Exception - Invalid inputs, DB save
	 * @Waring - Record exist
	 */
	public function return_ID_add_node_host($name,
	$parentNode,
	$treeId,
	$hostId,
	$hostGroupStyle){

		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_add_node_host","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_tree_add_node_host($name, $parentNode, $treeId, $hostId, $hostGroupStyle, $errors); //id, title
		return soapResponse($returnResult["id"], $errors);
	}

	/* Function add node graph to cacti menu
	 *
	 * @arg $parentNode - (num)
	 * @arg $treeId - (num)
	 * @arg $graphId - (num)
	 * @arg $rra_id - (num)
	 * @returns - $nodeId - (num)
	 * @Exception - Invalid inputs, DB save
	 * @Waring - Non
	 */
	public function return_ID_add_node_graph($parentNode, $treeId, $graphId, $rra_id){

		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_add_node_graph","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_tree_add_node_graph($parentNode, $treeId, $graphId, $rra_id, $errors);
		return soapResponse($returnResult["id"], $errors);
	}

	/* Function get Tree Nodes to cacti menu
	 *
	 * @arg $treeId - (num)
	 * @returns - $treeNodes["id"], $treeNodes["title"] - Array
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_list_IDTitle_get_TreeNodes($treeId)
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_list_IDTitle_get_TreeNodes","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_tree_get_TreeNodes($treeId, $errors); //id title

		return soapResponse($returnResult, $errors);
	}

	/* Function get list of all Trees
	 *
	 * @arg $name - (str)
	 * @returns - $tree["id"], $tree["name"] - (Array(Array))
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_list_IDName_get_Trees()
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_list_IDName_get_Trees","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_tree_get_Trees($errors); //id, name
		
		return soapResponse($returnResult, $errors);
	}

	/* Function get Tree id by Name
	 *
	 * @arg $name - (str)
	 * @returns - $treeId - (num)
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_ID_get_Tree_By_Name($name)
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_get_Tree_By_Name","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_tree_get_Tree_By_Name($name, $errors);//id, name
		return soapResponse($returnResult["id"], $errors);
	}

	/* Function get Tree Node id by Name
	 *
	 * @arg $treeId - (num)
	 * @arg $title - (str)
	 * @returns - $treeNodeId - (num)
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_ID_get_Tree_Node_By_Name($treeId, $title)
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_get_Tree_Node_By_Name","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_tree_get_Tree_Node_By_Name($treeId, $title, $errors);//id, title 
		return soapResponse($returnResult["id"], $errors);
	}


	//################################################

	/* Function add graph DS
	 *
	 * @arg $graphTitle - Optional, if value empty then v use Suggested Values - (str)
	 * @arg $template_id - id of graph id - (num)
	 * @arg $hostId - id of host to graph
	 * @arg $snmpQueryId
	 * @arg $snmpQueryTypeID
	 * @arg $snmpFieldName - name (oid)
	 * @arg $snmpValueName - value (oid value)
	 * @returns - $local_graph_id - return graph id (num)
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_ID_add_Graph_DS($graphTitle, $template_id,
	$hostId, $snmpQueryId, $snmpQueryTypeID, $snmpFieldName, $snmpValueName){

		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_add_Graph_DS","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_graph_add_Graph_DS($graphTitle, $template_id, $hostId, $snmpQueryId, $snmpQueryTypeID, $snmpFieldName, $snmpValueName, $errors);
		return soapResponse($returnResult, $errors);

	}

	/* Function add graph CG
	 *
	 * @arg $graphTitle - Optional, if value empty then v use Suggested Values - (str)
	 * @arg $template_id - id of graph id - (num)
	 * @arg $hostId - id of host to graph
	 * @returns - $local_graph_id - return graph id (num)
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_ID_add_Graph_CG($graphTitle, $template_id, $hostId){

		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_add_Graph_CG","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$empty = array();
		$returnResult = api_graph_add_Graph_CG($graphTitle, $template_id, $hostId, $empty, false, $errors);
		return soapResponse($returnResult, $errors);

	}

	/* Function check graph DS if exist
	 *
	 * @arg $template_id - id of graph id - (num)
	 * @arg $hostId - id of host to graph
	 * @arg $snmpQueryId
	 * @arg $snmpFieldName - name (oid)
	 * @arg $snmpValueName - value (oid value)
	 * @returns - $local_graph_id - return graph id (num)
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_ID_get_Graph_DS_By_Data($template_id, $hostId, $snmpQueryId, $snmpFieldName, $snmpValueName){

		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_get_Graph_DS_By_Data","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_graph_get_Graph_DS_By_Data($template_id, $hostId, $snmpQueryId, $snmpFieldName, $snmpValueName, $errors);
		return soapResponse($returnResult, $errors);
	}

	/* Function check graph CG if exist
	 *
	 * @arg $template_id - id of graph id - (num)
	 * @arg $hostId - id of host to graph
	 * @returns - $local_graph_id - return graph id (num)
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_ID_get_Graph_CG_By_Data($template_id, $hostId){

		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_get_Graph_CG_By_Data","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
			
		$returnResult = api_graph_get_Graph_CG_By_Data($template_id, $hostId, $errors);
		return soapResponse($returnResult, $errors);
	}

	/* Function update graph DC title
	 *
	 * @arg $graphTitle - Optional, if value empty then v use Suggested Values - (str)
	 * @arg $template_id - id of graph id - (num)
	 * @arg $hostId - id of host to graph
	 * @arg $snmpQueryId
	 * @arg $snmpFieldName - name (oid)
	 * @arg $snmpValueName - value (oid value)
	 * @returns - $local_graph_id - return graph id (num)
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_ID_update_Graph_Title_DS($graphTitle, $template_id, $hostId,
	$snmpQueryId, $snmpFieldName, $snmpValueName){

		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_update_Graph_Title_DS","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_graph_update_Graph_Title_DS($graphTitle, $template_id, $hostId, $snmpQueryId, $snmpFieldName, $snmpValueName, $errors);
		return soapResponse($returnResult, $errors);
	}

	/* Function update graph CG title
	 *
	 * @arg $graphTitle - Optional, if value empty then v use Suggested Values - (str)
	 * @arg $template_id - id of graph id - (num)
	 * @arg $hostId - id of host to graph
	 * @returns - $local_graph_id - return graph id (num)
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_ID_update_Graph_Title_CG($graphTitle, $template_id, $hostId){

		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_update_Graph_Title_CG","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_graph_update_Graph_Title_CG($graphTitle, $template_id, $hostId, $errors);
		return soapResponse($returnResult, $errors);
	}

	/* Function check if field name and value for host exist
	 *
	 * @arg $hostId - id of host to graph
	 * @arg $snmpFieldName - name (oid)
	 * @arg $snmpValueName - value (oid value)
	 * @returns - ["host_id", "field_name", "field_value"]
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_Field_get_SNMP_Value_by_Name($hostId, $snmpFieldName, $snmpValueName)
	{
		$errors = array();

		if(!$this->authenticated){
			catchException("", "", "return_Field_get_SNMP_Value_by_Name","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_graph_get_SNMP_Value_by_Name($hostId, $snmpFieldName, $snmpValueName, $errors);//host_id, field_name
		return soapResponse($returnResult["field_value"], $errors);
	}

	/* Function check if field value for host exist
	 *
	 * @arg $hostId - id of host to graph
	 * @arg $snmpFieldName - name (oid)
	 * @returns - ["host_id", "field_name", "field_value"]
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_list_Value_get_SNMP_Values($hostId, $snmpFieldName)
	{
		$errors = array();

		if(!$this->authenticated){
			catchException("", "", "return_list_Value_get_SNMP_Values","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$i = 1;
		$returnResult = api_graph_get_SNMP_Values($hostId, $snmpFieldName, $errors); //host_id, field_name, field_value

		return soapResponse($returnResult, $errors);
	}

	/* Function get list of all fields for selected host
	 *
	 * @arg $hostId - id of host to graph
	 * @returns - list ["host_id", "field_name"]
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_list_Field_get_SNMP_Fields($hostId)
	{
		$errors = array();

		if(!$this->authenticated){
			catchException("", "", "return_list_Field_get_SNMP_Fields","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_graph_get_SNMP_Fields($hostId, $errors); //host_id, field_name

		return soapResponse($returnResult, $errors);
	}

	/* Function check if feld for selected host exisist
	 *
	 * @arg $hostId - id of host to graph
	 * @arg $snmpFieldName - name of Field (oid)
	 * @returns - list ["host_id", "field_name"]
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_Name_get_SNMP_Field_By_Name($hostId, $snmpFieldName)
	{
		$errors = array();

		if(!$this->authenticated){
			catchException("", "", "return_Name_get_SNMP_Field_By_Name","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_graph_get_SNMP_Field_By_Name($hostId, $snmpFieldName, $errors);//host_id, field_name
		return soapResponse($returnResult["field_name"], $errors);
	}

	/* Function get list of all graph templates
	 *
	 * @returns - list ["id", "name"]
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_list_IDName_get_Graph_Templates()
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_list_IDName_get_Graph_Templates","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_graph_get_Graph_Templates($errors);
		
		return soapResponse($returnResult, $errors);
	}

	/* Function get graph template selected by name
	 *
	 * @arg - $graphTemplateName - name of graph template (str)
	 * @returns - id (num)
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_ID_get_Graph_Template_by_Name($graphTemplateName)
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_get_Graph_Template_by_Name","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_graph_get_Graph_Template_by_Name($graphTemplateName);//id, name
		return soapResponse($returnResult["id"], $errors);
	}
	
	/* Function get data template selected by name
	 *
	 * @arg - $dataTemplateName - name of data template (str)
	 * @returns - id (num)
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_ID_get_Data_Template_by_Name($dataTemplateName)
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_get_Data_Template_by_Name","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_data_get_Data_Template_by_Name($dataTemplateName);//id, name
		return soapResponse($returnResult["id"], $errors);
	}

	/* Function get query type id selected by query id and query type name
	 *
	 * @arg - $snmpQueryTypeName - query type name (str)
	 * @arg - $snmpQueryId - query name (str)
	 * @returns - $queryTypeID - (num)
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_ID_get_SNMP_Query_Type_By_Name($snmpQueryTypeName, $snmpQueryId)
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_get_SNMP_Query_Type_By_Name","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_graph_get_SNMP_Query_Type_By_Name($snmpQueryTypeName, $snmpQueryId, $errors); //id, name
		return soapResponse($returnResult["id"], $errors);
	}

	/* Function get list of all query types selected by query id
	 *
	 * @arg - $snmpQueryId - query name (str)
	 * @returns - list ["id", "name"]
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_list_IDName_get_SNMP_Query_Types($snmpQueryId)
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_list_IDName_get_SNMP_Query_Types","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}

		$returnResult = api_graph_get_SNMP_Query_Types($snmpQueryId, $errors);//id, name

		return soapResponse($returnResult, $errors);
	}

	/* Function get query by id selected by query name
	 *
	 * @arg - $snmpQueryId - query name (str)
	 * @returns - list ["id", "name"]
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_ID_get_SNMP_Query_By_Name($snmpQueryName)
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_get_SNMP_Query_By_Name","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_graph_get_SNMP_Query_By_Name($snmpQueryName, $errors); //id, name
		return soapResponse($returnResult["id"], $errors);
	}

	/* Function get list all of queries
	 *
	 * @returns - list ["id", "name"]
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_list_IDName_get_SNMP_Queries()
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_list_IDName_get_SNMP_Queries","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_graph_get_SNMP_Queries($errors); //id, name

		return soapResponse($returnResult, $errors);
	}

	/* Function get host id selected by host name
	 *
	 * @arg - $hostName - host name (str)
	 * @returns - ["id", "hostname"]
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_ID_get_Host($hostName)
	{
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_ID_get_Host","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		$returnResult = api_device_get_By_FQDN($hostName, $errors); //id, hostname, description
		return soapResponse($returnResult["id"], $errors);
	}
	
	/* Function replace link path to new link location
	 *
	 * @arg - $hostId - (str)
	 * @arg - $template_id - (num)
	 * @arg - $snmpFieldName - (str)
	 * @arg - $snmpValueName - (str)
	 * @arg - $data_source_path - (str)
	 * @returns - ["data_template_data.id", "data_source_path", "hostname", "data_template.id", "field_name", "field_value"]
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_data_replace_data_DS($hostId, $template_id, $snmpFieldName, $snmpValueName, $data_source_path){
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_data_replace_data_DS","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		
		$returnResult = api_replace_data_DS($hostId, $template_id, $snmpFieldName, $snmpValueName, $data_source_path, $errors);
		return soapResponse($returnResult, $errors);
	}
	
	/* Function replace rrd file to new link location
	 *
	 * @arg - $hostId - (str)
	 * @arg - $template_id - (num)
	 * @arg - $snmpFieldName - (str)
	 * @arg - $snmpValueName - (str)
	 * @arg - $old_data_source_path - (str)
	 * @arg - $old_data_source_file - (str)
	 * @returns - ["last_line", "resoult_exec"]
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_data_replace_rrd_file_DS($hostId, $template_id, $snmpFieldName, $snmpValueName, $data_source_path){
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_data_replace_rrd_file_DS","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		
		$returnResult = api_replace_rrd_file_DS($hostId, $template_id, $snmpFieldName, $snmpValueName, $old_data_source_path, $old_data_source_file, $errors);
		return soapResponse($returnResult, $errors);
	}	
	
	/* Function get data source path from selected arguments
	 *
	 * @arg - $fqdn - (str)
	 * @arg - $template_id - (num)
	 * @arg - $snmpFieldName - (str)
	 * @arg - $snmpValueName - (str)
	 * @returns - $data_source_path
	 * @Exception - Input validation, No DB data
	 * @Waring - Non
	 */
	public function return_path_get_data_source_path_DS($hostId, $template_id, $snmpFieldName, $snmpValueName){
		$errors = array();
			
		if(!$this->authenticated){
			catchException("", "", "return_data_replace_rrd_file_DS","Incorrect username and password combination", $errors);
			return soapResponse("", $errors);
		}
		
		$returnResult = api_get_data_DS($hostId, $template_id, $snmpFieldName, $snmpValueName, $errors);
		return soapResponse($returnResult["data_source_path"], $errors);
	}
		
	
}

$server = new SOAPServer(null, array('uri'=>'urn:pc_SOAP_return_soap'));
$server->setClass('pc_SOAP_return_soap');
$server->handle();

function soapResponse($result, $errors){

    if (!empty($errors)) {
    	$returnResult["Errors"] = $errors;
    	sysLogThrowException($errors);
    } else {
    	$returnResult["Result"] = $result;
    }
    return $returnResult;
}

function sysLogThrowException($errors){
	foreach($errors as $throwException){
		cacti_log("ERROR: " . $throwException["description"] . " - " . $throwException["function"] . "/" . $throwException["variable"] . "=" . $throwException["value"], false, "SOAP");
	}
}

?>